import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Point;
//import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.Arrays;
import java.util.Collections;

/**
 * Kepala ular, yang mengontrol seluruh gerakan ular.
 * Berwarna merah.
 * 
 * @author Keenan M. Gebze 
 * @version 1.3
 */
public class KepalaUlar extends Ular
{

    private Arah arah = Arah.KANAN;
    private ArrayList<BadanUlar> bagianBadan;

    /**
     * Membuat Kepala Ular baru.
     * 
     * Membuat gambar ular.
     * Mempersiapkan variabel bagianBadan.
     * Mengawali bagianBadan dengan 6 bagian badan.
     */
    public KepalaUlar() {
        /**
         * Membuat gambar untuk KepalaUlar ini.
         */
        GreenfootImage gambar = new GreenfootImage(20,20);
        gambar.setColor(Color.BLUE);
        gambar.fill();
        setImage(gambar);
        
        /**
         * Membuat list array untuk menunjukan jumlah badan kita.
         */
        bagianBadan = new ArrayList<BadanUlar>();
        for(int i=0; i<=6; i++) {
            bagianBadan.add((bagianBadan.size() > 0)?new BadanUlar(bagianBadan.get(bagianBadan.size()-1)):new BadanUlar(this));
        }
        
        bisaDiikuti = true;
    }
    
    /**
     * Fungsi ini dipanggil oleh Greenfoot ketika objek KepalaUlar masuk ke World w.
     * Setelah masuk ke World, kita hanya ingin memperbaharui badannya, tergantung dari 
     * ArrayList bagianBadan tadi.
     * 
     * Fungsi ini hanya dipanggil sekali, yaitu ketika objek ini masuk ke Dunia (World)
     */
    public void addedToWorld(World w) {
        perbaharuiBadan();
    }
    
     /**
     * Fungsi ini dipanggil oleh Greenfoot selalu.
     */
    public void act() 
    {
        if(Greenfoot.isKeyDown("up") && arah != Arah.BAWAH) {
           arah = Arah.ATAS;
        }else if(Greenfoot.isKeyDown("down") && arah != Arah.ATAS) {
           arah = Arah.BAWAH;
        }else if(Greenfoot.isKeyDown("left") && arah != Arah.KANAN) {
           arah = Arah.KIRI;
        }else if(Greenfoot.isKeyDown("right") && arah != Arah.KIRI) {
           arah = Arah.KANAN;
        }

        posisiAkhir = new Point(getX(), getY());
        
        move(arah);
        
        if(getOneObjectAtOffset(0,0, MakananUlar.class) != null) {
            MakananUlar makanan = (MakananUlar) getOneObjectAtOffset(0,0, MakananUlar.class); 
            makan(makanan);
        }else if(getOneObjectAtOffset(0,0, Ular.class) != null) {
            meledak();
        }
    } 
    
    public BisaDiikuti yangDiikuti() {
        return null;
    }
    
    private void meledak() {
        bisaDiikuti = false;
        Greenfoot.playSound("51464__smcameron__bombexplosion.wav");
        bagianBadan = new ArrayList<BadanUlar>();
        getWorld().removeObject(this);
    }
    
    /**
     * Memperbaharui badan tergantung dari ArrayList<BadanUlar> bagianBadan.
     */
    private void perbaharuiBadan() {
         for(BadanUlar bu : bagianBadan) {
             getWorld().addObject(bu, ((Actor)bu.yangDiikuti()).getX(), ((Actor)bu.yangDiikuti()).getY());
          }
    }
    
    /**
     * Makan makanan ular.
     * 
     * @param makanan makanan ular yang akan dimakan.
     */
    public void makan(MakananUlar makanan) {
        Greenfoot.playSound("20279_Koops_Apple_Crunch_16.wav"); //credit to freesound.org
        makanan.dimakan();
        bagianBadan.add(new BadanUlar(bagianBadan.get(bagianBadan.size()-1)));
        perbaharuiBadan();
    }
    
        
}